class LayerMetadata:
    def __init__(self, target_path, inferred_commands, group_name):
        self.target_path = target_path
        self.inferred_commands = inferred_commands
        self.group_name = group_name

